
#-----------------------------------------------------------------------
# Copyright (C) 2000-2001, Jean-Sebastien Morisset <jsmoriss@mvlan.net>
#-----------------------------------------------------------------------
# $Id: bitlen_to_netmask.sh,v 1.4 2001/08/11 17:01:20 jsmoriss Exp $
#-----------------------------------------------------------------------
#  $RCSfile: bitlen_to_netmask.sh,v $
#   $Author: jsmoriss $
# $Revision: 1.4 $
#     $Date: 2001/08/11 17:01:20 $
#   $Locker:  $ 
#     $Name:  $
#-----------------------------------------------------------------------
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by 
# the Free Software Foundation <http://www.gnu.org/copyleft/gpl.txt>.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License (LICENSE file in archive) for more details.
#
#-----------------------------------------------------------------------
# README
#-----------------------------------------------------------------------
#
# A bourne shell function used by rcf; a very secure, fast, and complete
# firewall for linux <http://jsmoriss.mvlan.net/linux/rcf.html>.
#
# Return the netmask using the bit length. Thanks to Paolo Bonzini 
# <bonzini@pc-amo3.elet.polimi.it> for contributing the sed code for
# this function.
#
# Example: Bitlen_To_Netmask 28
# Returns: 255.255.255.240
#
#-----------------------------------------------------------------------
# FUNCTION
#-----------------------------------------------------------------------

Bitlen_To_Netmask () {

	[ "$1" ] && { expr $1 / 8; expr $1 % 8; } | sed -ne ' 
		1 { 
			# Add the right number of bytes set to 255  
			s/[1234]$/255.&/ 
			s/[234]$/255.&/ 
			s/[34]$/255.&/ 
			s/4$/255.&/ 
			s/.$// 
			h 
			d 
		} 
 
		2 {
			s/^1$/128/
			s/^2$/192/
			s/^3$/224/
			s/^4$/240/
			s/^5$/248/
			s/^6$/252/
			s/^7$/254/
			H
			g
			s/\n//
 
			# Pad and remove the excess bytes
			s/$/.0.0.0.0/
			s/\([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\).*/\1/p
		}
	'
}

